//Software controle Motor de passo V0.1
//Controlador ESP32 - OPS010

//Inclusão das bibliotecas
#include "AccelStepper.h"

//Definição dos estados de máquina
#define ST_POWER              0
#define ST_HOME_BWD           10
#define ST_FIX_HOM_BWD        11
#define ST_FIX_HOM_BWD_STP    12
#define ST_HOME_FWD           20
#define ST_FIX_HOM_FWD        21
#define ST_FIX_HOM_FWD_STP    22
#define ST_WAIT_FWD           30
#define ST_FORWARD            31
#define ST_FIX_FWD            32
#define ST_STOP_FWD           33
#define ST_WAIT_BWD           40
#define ST_FIX_BWD            41
#define ST_BACKWARD           42
#define ST_STOP_BWD           43

//Mapeamento dos IOs e definição das Seriais
// Definição dos pinos do controlador
//Saidas digiais
#define pin_Y1 13
#define pin_Y2 12
#define pin_Y3 27
#define pin_Y4 26
#define pin_Y5 25
#define pin_Y6 33
#define pin_Y7 32
#define pin_Y8 4

//Entradas digitais
#define pin_X1 15
#define pin_X2 14
#define pin_X3 35
#define pin_X4 5
#define pin_X5 18
#define pin_X6 19
#define pin_X7 34
#define pin_X8 39

//Entradas analógicas
#define pin_AI1 36

//Saídas analógicas
#define pin_AO1 23

//Pinos auxiliares
#define ENTX_pin 2 //Pino de comunicação RS485

//Definição dos motores
//Define some steppers and the pins the will use
AccelStepper stepper1(AccelStepper::DRIVER, pin_Y1, pin_Y2);

//Definição das variáveis de controle
int INP_Comando = 0;
int INP_Sensor = 0;

int STA_Estado_Maquina = 0;

float PAR_Max_Speed = 10000.0;
float PAR_Home_Speed = 500.0;
float PAR_Acceleratiton = 40000.0;
long PAR_Max_Position = 100000;
long PAR_Deceleration = 2000;

long AUX_Position = 0;

void setup() {
  //Inicializa comunicações
  Serial.begin(115200); // (USB)
  //Serial1.begin(115200); // (TTL)
  //Serial2.begin(115200); //(485)

  //Inicializa hardware
  pinMode(pin_X1, INPUT);
  pinMode(pin_X2, INPUT);
  pinMode(pin_X3, INPUT);
  pinMode(pin_X4, INPUT);
  pinMode(pin_X5, INPUT);
  pinMode(pin_X6, INPUT);
  pinMode(pin_X7, INPUT);
  pinMode(pin_X8, INPUT);

  pinMode(pin_Y1, OUTPUT);
  pinMode(pin_Y2, OUTPUT);
  pinMode(pin_Y3, OUTPUT);
  pinMode(pin_Y4, OUTPUT);
  pinMode(pin_Y5, OUTPUT);
  pinMode(pin_Y6, OUTPUT);
  pinMode(pin_Y7, OUTPUT);
  pinMode(pin_Y8, OUTPUT);
  pinMode(ENTX_pin, OUTPUT);

  digitalWrite(pin_Y1, LOW);
  digitalWrite(pin_Y2, LOW);
  digitalWrite(pin_Y3, LOW);
  digitalWrite(pin_Y4, LOW);
  digitalWrite(pin_Y5, LOW);
  digitalWrite(pin_Y6, LOW);
  digitalWrite(pin_Y7, LOW);
  digitalWrite(pin_Y8, LOW);

  stepper1.setMaxSpeed(PAR_Max_Speed);
  stepper1.setAcceleration(PAR_Acceleratiton);

}

void loop() {

//***************************************************************************
//Leitura das entradas
//***************************************************************************
  INP_Comando = !digitalRead(pin_X1); //Botão de comando para movimento
  INP_Sensor = !digitalRead(pin_X2); //Usado para home e fim de curso

//***************************************************************************
//Máquina de estados
//***************************************************************************

  switch (STA_Estado_Maquina) {
//***************************************************************************
  case ST_POWER://Inicializa sistema
    stepper1.setMaxSpeed(PAR_Home_Speed);
    stepper1.moveTo(-PAR_Max_Position);
    STA_Estado_Maquina = ST_HOME_BWD;
  break;
//***************************************************************************
  case ST_HOME_BWD://Movimenta o eixo em direção a origem até o sensor
    if (INP_Sensor == 0){
      stepper1.run();
    }else{
      AUX_Position = stepper1.currentPosition();
      STA_Estado_Maquina = ST_FIX_HOM_BWD;
    }
  break;
//***************************************************************************
  case ST_FIX_HOM_BWD://Finaliza o movimento para zerar a posição atual
    stepper1.moveTo(AUX_Position);
    stepper1.run();
    if (stepper1.distanceToGo() == 0){
      STA_Estado_Maquina = ST_FIX_HOM_BWD_STP;
    }
  break;
//***************************************************************************
  case ST_FIX_HOM_BWD_STP://Realiza parada do motor
    stepper1.stop();
    stepper1.setCurrentPosition(0);
    stepper1.moveTo(PAR_Max_Position);
    STA_Estado_Maquina = ST_HOME_FWD;
  break;
//***************************************************************************
  case ST_HOME_FWD://Movimenta o eixo para frente até sair do sensor
    if (INP_Sensor == 1){
      stepper1.run();
    }else{
      AUX_Position = stepper1.currentPosition();
      STA_Estado_Maquina = ST_FIX_HOM_FWD;
    }
  break;
//***************************************************************************
  case ST_FIX_HOM_FWD://Finaliza o movimento para zerar a posição atual
    stepper1.moveTo(AUX_Position);
    stepper1.run();
    if (stepper1.distanceToGo() == 0){
      STA_Estado_Maquina = ST_FIX_HOM_FWD_STP;
    }
  break;
//***************************************************************************
  case ST_FIX_HOM_FWD_STP://Realiza parada do motor
    stepper1.stop();
    stepper1.setCurrentPosition(0);
    stepper1.setMaxSpeed(PAR_Max_Speed);
    STA_Estado_Maquina = ST_WAIT_FWD;
  break;
//***************************************************************************
  case ST_WAIT_FWD://Aguarda comando para movimentar para frente
    stepper1.moveTo(PAR_Max_Position);
    if (INP_Comando == 1){
      STA_Estado_Maquina = ST_FORWARD;
    }
  break;
//***************************************************************************
  case ST_FORWARD://Movimenta para frente durante ativação do comando
    if (INP_Comando == 1){
      stepper1.run();
    }else{
      AUX_Position = stepper1.currentPosition() + PAR_Deceleration;
      if (AUX_Position >= PAR_Max_Position){
        AUX_Position = PAR_Max_Position;
      }
      STA_Estado_Maquina = ST_FIX_FWD;
    }
  break;
//***************************************************************************
  case ST_FIX_FWD://Finaliza o movimento com desaceleração
    stepper1.moveTo(AUX_Position);
    stepper1.run();
    if (stepper1.distanceToGo() == 0){
      AUX_Position = stepper1.currentPosition();
      STA_Estado_Maquina = ST_STOP_FWD;
    }
  break;
//***************************************************************************
  case ST_STOP_FWD://Encerra o movimento para frente
    stepper1.stop();
    stepper1.setCurrentPosition(AUX_Position);
    STA_Estado_Maquina = ST_WAIT_BWD;
  break;
//***************************************************************************
  case ST_WAIT_BWD://Aguarda comando para movimentar para origem
    stepper1.moveTo(0);
    if (INP_Comando == 1){
      STA_Estado_Maquina = ST_BACKWARD;
    }
  break;
//***************************************************************************
  case ST_BACKWARD://Movimenta para origem durante ativação do comando ou sensor
    if ((INP_Comando == 1)&&(INP_Sensor == 0)){
      stepper1.run();
    }else{
      AUX_Position = stepper1.currentPosition() - PAR_Deceleration;
      if (AUX_Position <= 0){
        AUX_Position = 0;
      }
      STA_Estado_Maquina = ST_FIX_BWD;
    }
  break;
//***************************************************************************
  case ST_FIX_BWD://Finaliza o movimento com desaceleração
    stepper1.moveTo(AUX_Position);
    stepper1.run();
    if (stepper1.distanceToGo() == 0){
      STA_Estado_Maquina = ST_STOP_BWD;
    }
  break;
//***************************************************************************
  case ST_STOP_BWD://Encerra o movimento para origem
    stepper1.stop();
    stepper1.setCurrentPosition(AUX_Position);
      if (INP_Sensor == 0){
        STA_Estado_Maquina = ST_WAIT_FWD;
      }else{
        STA_Estado_Maquina = ST_POWER;
      }  
  break;
  }
}
